@@ -883,7 +883,6 @@ photos_application_startup (GApplication *application)
   priv->mode_cntrlr = photos_mode_controller_dup_singleton ();
 
   action = g_simple_action_new ("about", NULL);
-
   g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_about), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
@@ -914,15 +913,18 @@ photos_application_startup (GApplication *application)
   g_signal_connect_swapped (priv->properties_action, "activate", G_CALLBACK (photos_application_properties), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->properties_action));
 
-  priv->remote_display_action = g_simple_action_new ("remote-display-current", NULL);
-  g_signal_connect_swapped (priv->remote_display_action, "activate", G_CALLBACK (photos_application_remote_display_current), self);
-  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->remote_display_action));
-
   action = g_simple_action_new ("quit", NULL);
   g_signal_connect_swapped (action, "activate", G_CALLBACK (photos_application_quit), self);
   g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (action));
   g_object_unref (action);
 
+  priv->remote_display_action = g_simple_action_new ("remote-display-current", NULL);
+  g_signal_connect_swapped (priv->remote_display_action,
+                            "activate",
+                            G_CALLBACK (photos_application_remote_display_current),
+                            self);
+  g_action_map_add_action (G_ACTION_MAP (self), G_ACTION (priv->remote_display_action));
+
   state = g_variant_new ("b", FALSE);
   priv->search_action = g_simple_action_new_stateful ("search", NULL, state);
   g_signal_connect (priv->search_action, "activate", G_CALLBACK (photos_application_action_toggle), self);
