@@ -32,7 +32,6 @@
 #include <gio/gio.h>
 #include <glib/gi18n.h>
 
-#include "photos-collection-manager.h"
 #include "photos-embed.h"
 #include "photos-filterable.h"
 #include "photos-item-manager.h"
@@ -64,7 +63,6 @@ struct _PhotosEmbedSearchState
 struct _PhotosEmbedPrivate
 {
   GAction *search_action;
-  GCancellable *loader_cancellable;
   GtkWidget *collections;
   GtkWidget *favorites;
   GtkWidget *no_results;
@@ -77,7 +75,6 @@ struct _PhotosEmbedPrivate
   GtkWidget *stack;
   GtkWidget *stack_overlay;
   GtkWidget *toolbar;
-  PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
   PhotosBaseManager *src_mngr;
   PhotosBaseManager *srch_mngr;
@@ -210,34 +207,48 @@ photos_embed_prepare_for_preview (PhotosEmbed *self)
    */
 
   photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
+  photos_preview_view_set_node (PHOTOS_PREVIEW_VIEW (priv->preview), NULL);
   gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "preview");
 }
 
 
 static void
-photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user_data)
+photos_embed_load_finished (PhotosEmbed *self, PhotosBaseItem *item, GeglNode *node)
+{
+  PhotosEmbedPrivate *priv = self->priv;
+
+  photos_main_toolbar_set_cancellable (PHOTOS_MAIN_TOOLBAR (priv->toolbar), NULL);
+  photos_embed_clear_load_timer (self);
+  photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
+
+  if (node == NULL)
+    return;
+
+  photos_preview_view_set_node (PHOTOS_PREVIEW_VIEW (priv->preview), node);
+
+  /* TODO: set toolbar model */
+}
+
+
+static gboolean
+photos_embed_load_show_timeout (gpointer user_data)
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
   PhotosEmbedPrivate *priv = self->priv;
-  GError *error;
-  GeglNode *node;
-  GtkListStore *model;
-  GtkTreePath *current_path;
-  PhotosBaseItem *item = PHOTOS_BASE_ITEM (source_object);
+
+  priv->load_show_id = 0;
+  photos_spinner_box_start (PHOTOS_SPINNER_BOX (priv->spinner_box));
+  g_object_unref (self);
+  return G_SOURCE_REMOVE;
+}
+
+
+static void
+photos_embed_load_started (PhotosEmbed *self, PhotosBaseItem *item, GCancellable *cancellable)
+{
+  PhotosEmbedPrivate *priv = self->priv;
   PhotosWindowMode mode;
 
-  photos_embed_clear_load_timer (self);
-  g_clear_object (&priv->loader_cancellable);
-
-  error = NULL;
-  node = photos_base_item_load_finish (item, res, &error);
-  if (error != NULL)
-    {
-      g_warning ("Unable to load the item: %s", error->message);
-      g_error_free (error);
-      goto out;
-    }
-
   mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
 
   /* This is not needed when:
@@ -249,6 +260,8 @@ photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user
       || mode == PHOTOS_WINDOW_MODE_OVERVIEW
       || mode == PHOTOS_WINDOW_MODE_SEARCH)
     {
+      GtkListStore *model;
+      GtkTreePath *current_path;
       GtkWidget *view_container;
 
       view_container = photos_embed_get_view_container_from_mode (self, mode);
@@ -257,38 +270,13 @@ photos_embed_item_load (GObject *source_object, GAsyncResult *res, gpointer user
       photos_preview_view_set_model (PHOTOS_PREVIEW_VIEW (priv->preview), GTK_TREE_MODEL (model), current_path);
     }
 
-  photos_preview_view_set_node (PHOTOS_PREVIEW_VIEW (priv->preview), node);
-
-  /* TODO: set toolbar model */
-
-  /* If we are already in the preview and navigating using the
-   * buttons, then the window-mode-changed signal won't be fired. So
-   * we need to prepare it ourselves.
-   */
-  if (mode != PHOTOS_WINDOW_MODE_PREVIEW)
-    photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_PREVIEW);
-  else
-    photos_embed_prepare_for_preview (self);
-
+  photos_mode_controller_set_window_mode (priv->mode_cntrlr, PHOTOS_WINDOW_MODE_PREVIEW);
   photos_mode_controller_set_can_fullscreen (priv->mode_cntrlr, TRUE);
 
- out:
-  g_clear_object (&node);
-  g_object_unref (self);
-}
+  photos_main_toolbar_set_cancellable (PHOTOS_MAIN_TOOLBAR (priv->toolbar), cancellable);
 
-
-static gboolean
-photos_embed_load_show_timeout (gpointer user_data)
-{
-  PhotosEmbed *self = PHOTOS_EMBED (user_data);
-  PhotosEmbedPrivate *priv = self->priv;
-
-  priv->load_show_id = 0;
-  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "spinner");
-  photos_spinner_box_start (PHOTOS_SPINNER_BOX (priv->spinner_box));
-  g_object_unref (self);
-  return G_SOURCE_REMOVE;
+  photos_embed_clear_load_timer (self);
+  self->priv->load_show_id = g_timeout_add (400, photos_embed_load_show_timeout, g_object_ref (self));
 }
 
 
@@ -331,8 +319,8 @@ photos_embed_active_changed (PhotosBaseManager *manager, GObject *object, gpoint
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
   PhotosEmbedPrivate *priv = self->priv;
+  PhotosBaseItem *active_collection;
   PhotosWindowMode mode;
-  GObject *active_collection;
   GObject *active_item;
   GVariant *state;
   gboolean show_search;
@@ -341,7 +329,7 @@ photos_embed_active_changed (PhotosBaseManager *manager, GObject *object, gpoint
    * preview or collection viewin. Restore it when we are back.
    */
 
-  active_collection = photos_base_manager_get_active_object (priv->col_mngr);
+  active_collection = photos_item_manager_get_active_collection (PHOTOS_ITEM_MANAGER (priv->item_mngr));
   active_item = photos_base_manager_get_active_object (priv->item_mngr);
   mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
   show_search = (mode == PHOTOS_WINDOW_MODE_PREVIEW && active_item == NULL && active_collection == NULL)
@@ -354,61 +342,6 @@ photos_embed_active_changed (PhotosBaseManager *manager, GObject *object, gpoint
 
   state = g_variant_new ("b", show_search);
   g_action_change_state (priv->search_action, state);
-
-  if (object == NULL)
-    return;
-
-  photos_embed_clear_load_timer (self);
-
-  if (photos_base_item_is_collection (PHOTOS_BASE_ITEM (object)))
-    return;
-
-  priv->load_show_id = g_timeout_add (400, photos_embed_load_show_timeout, g_object_ref (self));
-
-  priv->loader_cancellable = g_cancellable_new ();
-  photos_base_item_load_async (PHOTOS_BASE_ITEM (object),
-                               priv->loader_cancellable,
-                               photos_embed_item_load,
-                               g_object_ref (self));
-}
-
-
-static void
-photos_embed_restore_last_page (PhotosEmbed *self)
-{
-  PhotosEmbedPrivate *priv = self->priv;
-  PhotosWindowMode mode;
-  const gchar *page;
-
-  mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
-  switch (mode)
-    {
-    case PHOTOS_WINDOW_MODE_COLLECTIONS:
-      page = "collections";
-      break;
-
-    case PHOTOS_WINDOW_MODE_FAVORITES:
-      page = "favorites";
-      break;
-
-    case PHOTOS_WINDOW_MODE_OVERVIEW:
-      page = "overview";
-      break;
-
-    case PHOTOS_WINDOW_MODE_PREVIEW:
-      page = "preview";
-      break;
-
-    case PHOTOS_WINDOW_MODE_SEARCH:
-      page = "search";
-      break;
-
-    default:
-      g_assert_not_reached ();
-      break;
-    }
-
-  gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), page);
 }
 
 
@@ -455,13 +388,6 @@ photos_embed_prepare_for_collections (PhotosEmbed *self)
   PhotosEmbedPrivate *priv = self->priv;
 
   photos_base_manager_set_active_object (priv->item_mngr, NULL);
-
-  if (priv->loader_cancellable != NULL)
-    {
-      g_cancellable_cancel (priv->loader_cancellable);
-      g_clear_object (&priv->loader_cancellable);
-    }
-
   photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
   gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "collections");
 }
@@ -473,13 +399,6 @@ photos_embed_prepare_for_favorites (PhotosEmbed *self)
   PhotosEmbedPrivate *priv = self->priv;
 
   photos_base_manager_set_active_object (priv->item_mngr, NULL);
-
-  if (priv->loader_cancellable != NULL)
-    {
-      g_cancellable_cancel (priv->loader_cancellable);
-      g_clear_object (&priv->loader_cancellable);
-    }
-
   photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
   gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "favorites");
 }
@@ -491,13 +410,6 @@ photos_embed_prepare_for_overview (PhotosEmbed *self)
   PhotosEmbedPrivate *priv = self->priv;
 
   photos_base_manager_set_active_object (priv->item_mngr, NULL);
-
-  if (priv->loader_cancellable != NULL)
-    {
-      g_cancellable_cancel (priv->loader_cancellable);
-      g_clear_object (&priv->loader_cancellable);
-    }
-
   photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
   gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "overview");
 }
@@ -509,13 +421,6 @@ photos_embed_prepare_for_search (PhotosEmbed *self)
   PhotosEmbedPrivate *priv = self->priv;
 
   photos_base_manager_set_active_object (priv->item_mngr, NULL);
-
-  if (priv->loader_cancellable != NULL)
-    {
-      g_cancellable_cancel (priv->loader_cancellable);
-      g_clear_object (&priv->loader_cancellable);
-    }
-
   photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
   gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "search");
 }
@@ -527,15 +432,9 @@ photos_embed_query_status_changed (PhotosEmbed *self, gboolean querying)
   PhotosEmbedPrivate *priv = self->priv;
 
   if (querying)
-    {
-      photos_spinner_box_start (PHOTOS_SPINNER_BOX (priv->spinner_box));
-      gtk_stack_set_visible_child_name (GTK_STACK (priv->stack), "spinner");
-    }
+    photos_spinner_box_start (PHOTOS_SPINNER_BOX (priv->spinner_box));
   else
-    {
-      photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
-      photos_embed_restore_last_page (self);
-    }
+    photos_spinner_box_stop (PHOTOS_SPINNER_BOX (priv->spinner_box));
 }
 
 
@@ -649,8 +548,6 @@ photos_embed_dispose (GObject *object)
   photos_embed_clear_search (self);
 
   g_clear_object (&priv->ntfctn_mngr);
-  g_clear_object (&priv->loader_cancellable);
-  g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
   g_clear_object (&priv->srch_mngr);
@@ -750,7 +647,7 @@ photos_embed_init (PhotosEmbed *self)
   gtk_stack_add_named (GTK_STACK (priv->stack), priv->preview, "preview");
 
   priv->spinner_box = photos_spinner_box_new ();
-  gtk_stack_add_named (GTK_STACK (priv->stack), priv->spinner_box, "spinner");
+  gtk_overlay_add_overlay (GTK_OVERLAY (priv->stack_overlay), priv->spinner_box);
 
   /* TODO: SearchBar.Dropdown, …
    */
@@ -775,11 +672,10 @@ photos_embed_init (PhotosEmbed *self)
                             G_CALLBACK (photos_embed_query_status_changed),
                             self);
 
-  priv->col_mngr = g_object_ref (state->col_mngr);
-  g_signal_connect (priv->col_mngr, "active-changed", G_CALLBACK (photos_embed_active_changed), self);
-
-  priv->item_mngr = photos_item_manager_dup_singleton ();
+  priv->item_mngr = g_object_ref (state->item_mngr);
   g_signal_connect (priv->item_mngr, "active-changed", G_CALLBACK (photos_embed_active_changed), self);
+  g_signal_connect_swapped (priv->item_mngr, "load-finished", G_CALLBACK (photos_embed_load_finished), self);
+  g_signal_connect_swapped (priv->item_mngr, "load-started", G_CALLBACK (photos_embed_load_started), self);
 
   priv->src_mngr = g_object_ref (state->src_mngr);
   g_signal_connect_swapped (priv->src_mngr, "active-changed", G_CALLBACK (photos_embed_search_changed), self);
